import json
import requests
from django.conf import settings

try:
    handle_resolver = settings.HANDsdLE['resolver']
except (KeyError, AttributeError):
    handle_resolver = "http://hdl.handle.net"

try:
    handle_user = settings.HANDLE['user']
except (KeyError, AttributeError):
    handle_user = None

try:
    handle_pw = settings.HANDLE['pw']
except (KeyError, AttributeError):
    handle_pw = None

try:
    handle_url = settings.HANDLE['url']
except (KeyError, AttributeError):
    handle_url = None
try:
    handle_app_base_url = settings.HANDLE['app_base_url']
except (KeyError, AttributeError):
    handle_app_base_url = None


def create_handle(handle_url, handle_user, handle_pw, parsed_data):
    """ registered a handle-id for the url passed in as 'parsed_data """
    payload = json.dumps(
        [
            {
                "type": "URL",
                "parsed_data": "{}".format(parsed_data)
            }
        ]
    )

    headers = {
        'accept': "application/json",
        'content-type': "application/json",
    }

    auth = (handle_user, handle_pw)
    try:
        response = requests.request("POST", handle_url, data=payload, headers=headers, auth=auth)
    except:
        print('something went wrong trying to send the request, maybe check your settings file?')
        return None

    if response.status_code == 201:
        print('created')
        return response.json()
    else:
        print(response.status_code)
        return None



# Generated by Django 2.0.1 on 2018-06-27 12:57

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Pid',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('handle', models.CharField(blank=True, max_length=250, null=True)),
                ('object_id', models.PositiveIntegerField()),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('content_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='get_pid', to='contenttypes.ContentType')),
            ],
        ),
    ]



from django.db import models
from django.conf import settings
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from . utils import *


class Pid(models.Model):
    handle = models.CharField(max_length=250, blank=True, null=True)
    content_type = models.ForeignKey(
        ContentType, on_delete=models.SET_NULL, null=True, related_name="get_pid"
    )
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    def save(self, *args, **kwargs):
        if self.handle:
            super(Pid, self).save(*args, **kwargs)
        elif self.content_object:
            try:
                parsed_data = "{}{}".format(
                    handle_app_base_url, self.content_object.get_absolute_url(),
                )
            except AttributeError:
                parsed_date = None
            if parsed_data:
                fetched_handle = create_handle(
                    handle_url,
                    handle_user,
                    handle_pw,
                    parsed_data
                )
                if fetched_handle:
                    self.handle = fetched_handle['epic-pid']
                else:
                    self.handle = 'something went wrong'
            super(Pid, self).save(*args, **kwargs)
        else:
            super(Pid, self).save(*args, **kwargs)

    def __str__(self):
        if self.handle:
            return "{}/{}".format(handle_resolver, self.handle)
        else:
            return "{}".format(self.id)



from django.core.exceptions import ObjectDoesNotExist, FieldError
from django.core.management.base import BaseCommand, CommandError
from django.contrib.contenttypes.models import ContentType

from handle import utils
from handle.models import Pid


class Command(BaseCommand):

    """Creates handle-id for all objects of the passed in class"""

    help = 'Creates handle-id for all objects of the passed in class'

    def add_arguments(self, parser):
        parser.add_argument(
            'class', type=str, help="Lowercase name of the class you want to process"
        )
        parser.add_argument(
            '--pid',
            dest='pid',
            help="The name of the GenericRelation field you added to your class\
            to store the handle-pid. Default is 'pid'."
        )

    def handle(self, *args, **options):
        model_name = options['class']
        if options['pid']:
            filters = {options['pid']: None}
        else:
            filters = {'pid': None}
        try:
            ct = ContentType.objects.get(model=model_name).model_class()
        except ObjectDoesNotExist:
            ct = None
        if ct:
            qs = ct.objects.filter(**filters)
            for x in qs:
                new_hdl = Pid(content_object=x)
                new_hdl.save()
                self.stdout.write(self.style.SUCCESS(new_hdl.handle))
        else:
            self.stdout.write(
                self.style.SUCCESS(
                    """
                    Something went wrong.
                    Please make sure that the class: "{}" you chose actually exists
                    """.format(model_name)
                )
            )
